@@ -23,6 +23,7 @@
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <sys/wait.h>
+#include <sys/resource.h>
 
 #include <fcntl.h>
 #include <err.h>
@@ -93,6 +94,7 @@ struct gotd_client {
 	struct event			 tmo;
 	uid_t				 euid;
 	gid_t				 egid;
+	char				*username;
 	struct gotd_child_proc		*repo;
 	struct gotd_child_proc		*auth;
 	struct gotd_child_proc		*session;
@@ -125,6 +127,16 @@ usage(void)
 	exit(1);
 }
 
+static void
+drop_privs(struct passwd *pw)
+{
+	/* Drop root privileges. */
+	if (setgid(pw->pw_gid) == -1)
+		fatal("setgid %d failed", pw->pw_gid);
+	if (setuid(pw->pw_uid) == -1)
+		fatal("setuid %d failed", pw->pw_uid);
+}
+
 static int
 unix_socket_listen(const char *unix_socket_path, uid_t uid, gid_t gid)
 {
@@ -371,6 +383,7 @@ disconnect(struct gotd_client *client)
 		close(client->fd);
 	else if (client->iev.ibuf.fd != -1)
 		close(client->iev.ibuf.fd);
+	free(client->username);
 	free(client);
 	client_cnt--;
 }
@@ -669,7 +682,7 @@ recv_connect(uint32_t *client_id, struct imsg *imsg)
 		return got_error(GOT_ERR_PRIVSEP_LEN);
 	memcpy(&iconnect, imsg->data, sizeof(iconnect));
 
-	s = imsg->fd;
+	s = imsg_get_fd(imsg);
 	if (s == -1) {
 		err = got_error(GOT_ERR_PRIVSEP_NO_FD);
 		goto done;
@@ -933,7 +946,7 @@ verify_imsg_src(struct gotd_client *client, struct gotd_child_proc *proc,
 			err = got_error_fmt(GOT_ERR_BAD_PACKET,
 			    "new connection for uid %d from PID %d "
 			    "which is not the listen process",
-			    proc->pid, client->euid);
+			    client->euid, proc->pid);
 		} else
 			ret = 1;
 		break;
@@ -942,7 +955,7 @@ verify_imsg_src(struct gotd_client *client, struct gotd_child_proc *proc,
 			err = got_error_fmt(GOT_ERR_BAD_PACKET,
 			    "authentication of uid %d from PID %d "
 			    "which is not the auth process",
-			    proc->pid, client->euid);
+			    client->euid, proc->pid);
 		} else
 			ret = 1;
 		break;
@@ -1133,6 +1146,7 @@ gotd_dispatch_auth_child(int fd, short event, void *arg)
 	struct imsg imsg;
 	uint32_t client_id = 0;
 	int do_disconnect = 0;
+	size_t datalen;
 
 	client = find_client_by_proc_fd(fd);
 	if (client == NULL) {
@@ -1176,13 +1190,18 @@ gotd_dispatch_auth_child(int fd, short event, void *arg)
 
 	evtimer_del(&client->tmo);
 
+	datalen = imsg.hdr.len - IMSG_HEADER_SIZE;
+
 	switch (imsg.hdr.type) {
 	case GOTD_IMSG_ERROR:
 		do_disconnect = 1;
 		err = gotd_imsg_recv_error(&client_id, &imsg);
 		break;
 	case GOTD_IMSG_ACCESS_GRANTED:
-		client->state = GOTD_CLIENT_STATE_ACCESS_GRANTED;
+		if (client->state != GOTD_CLIENT_STATE_NEW) {
+			do_disconnect = 1;
+			err = got_error(GOT_ERR_PRIVSEP_MSG);
+		}
 		break;
 	default:
 		do_disconnect = 1;
@@ -1195,16 +1214,26 @@ gotd_dispatch_auth_child(int fd, short event, void *arg)
 		log_debug("dropping imsg type %d from PID %d",
 		    imsg.hdr.type, client->auth->pid);
 	}
-	imsg_free(&imsg);
 
 	if (do_disconnect) {
 		if (err)
 			disconnect_on_error(client, err);
 		else
 			disconnect(client);
+		imsg_free(&imsg);
 		return;
 	}
 
+	client->state = GOTD_CLIENT_STATE_ACCESS_GRANTED;
+	if (datalen > 0)
+		client->username = strndup(imsg.data, datalen);
+	imsg_free(&imsg);
+	if (client->username == NULL &&
+	    asprintf(&client->username, "uid %d", client->euid) == -1) {
+		err = got_error_from_errno("asprintf");
+		goto done;
+	}
+
 	repo = gotd_find_repo_by_name(client->auth->repo_name, &gotd);
 	if (repo == NULL) {
 		err = got_error(GOT_ERR_NOT_GIT_REPO);
@@ -1212,8 +1241,8 @@ gotd_dispatch_auth_child(int fd, short event, void *arg)
 	}
 	kill_auth_proc(client);
 
-	log_info("authenticated uid %d for repository %s",
-	    client->euid, repo->name);
+	log_info("authenticated %s for repository %s",
+	    client->username, repo->name);
 
 	err = start_session_child(client, repo, gotd.argv0,
 	    gotd.confpath, gotd.daemonize, gotd.verbosity);
@@ -1823,13 +1852,15 @@ apply_unveil_selfexec(void)
 }
 
 static void
-drop_privs(struct passwd *pw)
+set_max_datasize(void)
 {
-	/* Drop root privileges. */
-	if (setgid(pw->pw_gid) == -1)
-		fatal("setgid %d failed", pw->pw_gid);
-	if (setuid(pw->pw_uid) == -1)
-		fatal("setuid %d failed", pw->pw_uid);
+	struct rlimit rl;
+
+	if (getrlimit(RLIMIT_DATA, &rl) != 0)
+		return;
+
+	rl.rlim_cur = rl.rlim_max;
+	setrlimit(RLIMIT_DATA, &rl);
 }
 
 int
@@ -2045,6 +2076,7 @@ main(int argc, char **argv)
 		/* NOTREACHED */
 		break;
 	case PROC_REPO_READ:
+		set_max_datasize();
 #ifndef PROFILE
 		if (pledge("stdio rpath recvfd unveil", NULL) == -1)
 			err(1, "pledge");
@@ -2065,6 +2097,7 @@ main(int argc, char **argv)
 		/* NOTREACHED */
 		exit(0);
 	case PROC_REPO_WRITE:
+		set_max_datasize();
 #ifndef PROFILE
 		if (pledge("stdio rpath recvfd unveil", NULL) == -1)
 			err(1, "pledge");
