@@ -59,7 +59,6 @@ static const struct querystring_keys querystring_keys[] = {
 	{ "headref",		HEADREF },
 	{ "index_page",		INDEX_PAGE },
 	{ "path",		PATH },
-	{ "page",		PAGE },
 };
 
 static const struct action_keys action_keys[] = {
@@ -71,6 +70,7 @@ static const struct action_keys action_keys[] = {
 	{ "diff",	DIFF },
 	{ "error",	ERR },
 	{ "index",	INDEX },
+	{ "patch",	PATCH },
 	{ "summary",	SUMMARY },
 	{ "tag",	TAG },
 	{ "tags",	TAGS },
@@ -186,7 +186,8 @@ gotweb_process_request(struct request *c)
 	 */
 
 	if (qs->action == BLAME || qs->action == BLOB ||
-	    qs->action == BLOBRAW || qs->action == DIFF) {
+	    qs->action == BLOBRAW || qs->action == DIFF ||
+	    qs->action == PATCH) {
 		if (qs->commit == NULL) {
 			error = got_error(GOT_ERR_BAD_QUERYSTRING);
 			goto err;
@@ -204,18 +205,27 @@ gotweb_process_request(struct request *c)
 	}
 
 	if (qs->action == BLOBRAW || qs->action == BLOB) {
+		if (qs->folder == NULL || qs->file == NULL) {
+			error = got_error(GOT_ERR_BAD_QUERYSTRING);
+			goto err;
+		}
+
 		error = got_get_repo_commits(c, 1);
 		if (error)
 			goto err;
 
 		error = got_open_blob_for_output(&c->t->blob, &c->t->fd,
-		    &binary, c);
+		    &binary, c, qs->folder, qs->file, qs->commit);
 		if (error)
 			goto err;
 	}
 
 	switch (qs->action) {
 	case BLAME:
+		if (qs->folder == NULL || qs->file == NULL) {
+			error = got_error(GOT_ERR_BAD_QUERYSTRING);
+			goto err;
+		}
 		error = got_get_repo_commits(c, 1);
 		if (error) {
 			log_warnx("%s: %s", __func__, error->msg);
@@ -229,7 +239,6 @@ gotweb_process_request(struct request *c)
 		if (binary) {
 			struct gotweb_url url = {
 				.index_page = -1,
-				.page = -1,
 				.action = BLOBRAW,
 				.path = qs->path,
 				.commit = qs->commit,
@@ -313,6 +322,21 @@ gotweb_process_request(struct request *c)
 			return;
 		gotweb_render_page(c->tp, gotweb_render_index);
 		return;
+	case PATCH:
+		error = got_get_repo_commits(c, 1);
+		if (error) {
+			log_warnx("%s: %s", __func__, error->msg);
+			goto err;
+		}
+		error = got_open_diff_for_output(&c->t->fp, c);
+		if (error) {
+			log_warnx("%s: %s", __func__, error->msg);
+			goto err;
+		}
+		if (gotweb_reply(c, 200, "text/plain", NULL) == -1)
+			return;
+		gotweb_render_patch(c->tp);
+		return;
 	case RSS:
 		error = got_get_repo_tags(c, D_MAXSLCOMMDISP);
 		if (error)
@@ -330,11 +354,11 @@ gotweb_process_request(struct request *c)
 			    error->msg);
 			goto err;
 		}
-		error = got_get_repo_commits(c, D_MAXSLCOMMDISP);
+		error = got_get_repo_commits(c, srv->summary_commits_display);
 		if (error)
 			goto err;
 		qs->action = TAGS;
-		error = got_get_repo_tags(c, D_MAXSLCOMMDISP);
+		error = got_get_repo_tags(c, srv->summary_tags_display);
 		if (error) {
 			log_warnx("%s: got_get_repo_tags: %s", __func__,
 			    error->msg);
@@ -442,11 +466,6 @@ gotweb_init_querystring(struct querystring **qs)
 	}
 
 	(*qs)->action = INDEX;
-	(*qs)->commit = NULL;
-	(*qs)->file = NULL;
-	(*qs)->folder = NULL;
-	(*qs)->index_page = 0;
-	(*qs)->path = NULL;
 
 	return error;
 }
@@ -563,7 +582,7 @@ gotweb_assign_querystring(struct querystring **qs, char *key, char *value)
 
 		switch (querystring_keys[el_cnt].element) {
 		case ACTION:
-			for (a_cnt = 0; a_cnt < ACTIONS__MAX; a_cnt++) {
+			for (a_cnt = 0; a_cnt < nitems(action_keys); a_cnt++) {
 				if (strcmp(value, action_keys[a_cnt].name) != 0)
 					continue;
 				else if (strcmp(value,
@@ -630,19 +649,6 @@ qa_found:
 				goto done;
 			}
 			break;
-		case PAGE:
-			if (*value == '\0')
-				break;
-			(*qs)->page = strtonum(value, INT64_MIN,
-			    INT64_MAX, &errstr);
-			if (errstr) {
-				error = got_error_from_errno3(__func__,
-				    "strtonum", errstr);
-				goto done;
-			}
-			if ((*qs)->page < 0)
-				(*qs)->page = 0;
-			break;
 		}
 
 		/* entry found */
@@ -727,8 +733,7 @@ gotweb_free_transport(struct transport *t)
 	gotweb_free_repo_dir(t->repo_dir);
 	gotweb_free_querystring(t->qs);
 	free(t->more_id);
-	free(t->next_id);
-	free(t->prev_id);
+	free(t->tags_more_id);
 	if (t->blob)
 		got_object_blob_close(t->blob);
 	if (t->fp) {
@@ -750,7 +755,7 @@ gotweb_free_transport(struct transport *t)
 }
 
 void
-gotweb_get_navs(struct request *c, struct gotweb_url *prev, int *have_prev,
+gotweb_index_navs(struct request *c, struct gotweb_url *prev, int *have_prev,
     struct gotweb_url *next, int *have_next)
 {
 	struct transport *t = c->t;
@@ -759,52 +764,21 @@ gotweb_get_navs(struct request *c, struct gotweb_url *prev, int *have_prev,
 
 	*have_prev = *have_next = 0;
 
-	switch(qs->action) {
-	case INDEX:
-		if (qs->index_page > 0) {
-			*have_prev = 1;
-			*prev = (struct gotweb_url){
-				.action = -1,
-				.index_page = qs->index_page - 1,
-				.page = -1,
-			};
-		}
-		if (t->next_disp == srv->max_repos_display &&
-		    t->repos_total != (qs->index_page + 1) *
-		    srv->max_repos_display) {
-			*have_next = 1;
-			*next = (struct gotweb_url){
-				.action = -1,
-				.index_page = qs->index_page + 1,
-				.page = -1,
-			};
-		}
-		break;
-	case TAGS:
-		if (t->prev_id && qs->commit != NULL &&
-		    strcmp(qs->commit, t->prev_id) != 0) {
-			*have_prev = 1;
-			*prev = (struct gotweb_url){
-				.action = TAGS,
-				.index_page = -1,
-				.page = qs->page - 1,
-				.path = qs->path,
-				.commit = t->prev_id,
-				.headref = qs->headref,
-			};
-		}
-		if (t->next_id) {
-			*have_next = 1;
-			*next = (struct gotweb_url){
-				.action = TAGS,
-				.index_page = -1,
-				.page = qs->page + 1,
-				.path = qs->path,
-				.commit = t->next_id,
-				.headref = qs->headref,
-			};
-		}
-		break;
+	if (qs->index_page > 0) {
+		*have_prev = 1;
+		*prev = (struct gotweb_url){
+			.action = -1,
+			.index_page = qs->index_page - 1,
+		};
+	}
+	if (t->next_disp == srv->max_repos_display &&
+	    t->repos_total != (qs->index_page + 1) *
+	    srv->max_repos_display) {
+		*have_next = 1;
+		*next = (struct gotweb_url){
+			.action = -1,
+			.index_page = qs->index_page + 1,
+		};
 	}
 }
 
@@ -826,9 +800,6 @@ gotweb_render_index(struct template *tp)
 		return -1;
 
 	for (d_i = 0; d_i < t->nrepos; d_i++) {
-		if (srv->max_repos > 0 && t->prev_disp == srv->max_repos)
-			break;
-
 		if (strcmp(sd_dent[d_i]->d_name, ".") == 0 ||
 		    strcmp(sd_dent[d_i]->d_name, "..") == 0) {
 			d_skipped++;
@@ -882,11 +853,7 @@ gotweb_render_index(struct template *tp)
 	}
 	t->repos_total = t->nrepos - d_skipped;
 
-	if (srv->max_repos_display == 0)
-		return 0;
-	if (srv->max_repos > 0 && srv->max_repos < srv->max_repos_display)
-		return 0;
-	if (t->repos_total <= srv->max_repos ||
+	if (srv->max_repos_display == 0 ||
 	    t->repos_total <= srv->max_repos_display)
 		return 0;
 
@@ -986,6 +953,8 @@ gotweb_action_name(int action)
 		return "err";
 	case INDEX:
 		return "index";
+	case PATCH:
+		return "patch";
 	case SUMMARY:
 		return "summary";
 	case TAG:
@@ -1084,12 +1053,6 @@ gotweb_render_url(struct request *c, struct gotweb_url *url)
 		sep = "&";
 	}
 
-	if (url->page != -1) {
-		if (tp_writef(c->tp, "%spage=%d", sep, url->page) == -1)
-			return -1;
-		sep = "&";
-	}
-
 	return 0;
 }
 
@@ -1125,28 +1088,20 @@ gotweb_load_got_path(struct request *c, struct repo_dir *repo_dir)
 	dt = opendir(dir_test);
 	if (dt == NULL) {
 		free(dir_test);
-	} else {
-		repo_dir->path = dir_test;
-		dir_test = NULL;
-		goto open_repo;
+		if (asprintf(&dir_test, "%s/%s", srv->repos_path,
+		    repo_dir->name) == -1)
+			return got_error_from_errno("asprintf");
+		dt = opendir(dir_test);
+		if (dt == NULL) {
+			free(dir_test);
+			return got_error_path(repo_dir->name,
+			    GOT_ERR_NOT_GIT_REPO);
+		}
 	}
 
-	if (asprintf(&dir_test, "%s/%s", srv->repos_path,
-	    repo_dir->name) == -1) {
-		error = got_error_from_errno("asprintf");
-		goto err;
-	}
+	repo_dir->path = dir_test;
+	dir_test = NULL;
 
-	dt = opendir(dir_test);
-	if (dt == NULL) {
-		error = got_error_path(repo_dir->name, GOT_ERR_NOT_GIT_REPO);
-		goto err;
-	} else {
-		repo_dir->path = dir_test;
-		dir_test = NULL;
-	}
-
-open_repo:
 	if (srv->respect_exportok &&
 	    faccessat(dirfd(dt), "git-daemon-export-ok", F_OK, 0) == -1) {
 		error = got_error_path(repo_dir->name, GOT_ERR_NOT_GIT_REPO);
