@@ -77,10 +77,10 @@
 #define D_SHOWDESC		 1
 #define D_SHOWURL		 1
 #define D_RESPECTEXPORTOK	 0
-#define D_MAXREPO		 0
 #define D_MAXREPODISP		 25
 #define D_MAXSLCOMMDISP		 10
 #define D_MAXCOMMITDISP		 25
+#define D_MAXSLTAGDISP		 3
 
 #define BUF			 8192
 
@@ -204,8 +204,7 @@ struct transport {
 	struct repo_dir		*repo_dir;
 	struct querystring	*qs;
 	char			*more_id;
-	char			*next_id;
-	char			*prev_id;
+	char			*tags_more_id;
 	unsigned int		 repos_total;
 	unsigned int		 next_disp;
 	unsigned int		 prev_disp;
@@ -300,9 +299,10 @@ struct server {
 	char		 logo_url[GOTWEBD_MAXTEXT];
 	char		 custom_css[PATH_MAX];
 
-	size_t		 max_repos;
 	size_t		 max_repos_display;
 	size_t		 max_commits_display;
+	size_t		 summary_commits_display;
+	size_t		 summary_tags_display;
 
 	int		 show_site_owner;
 	int		 show_repo_owner;
@@ -385,7 +385,6 @@ struct gotwebd {
 struct gotweb_url {
 	int		 action;
 	int		 index_page;
-	int		 page;
 	const char	*commit;
 	const char	*previd;
 	const char	*prevset;
@@ -405,7 +404,6 @@ struct querystring {
 	char		*headref;
 	int		 index_page;
 	char		*path;
-	int		 page;
 };
 
 struct querystring_keys {
@@ -426,7 +424,6 @@ enum querystring_elements {
 	HEADREF,
 	INDEX_PAGE,
 	PATH,
-	PAGE,
 };
 
 enum query_actions {
@@ -438,12 +435,12 @@ enum query_actions {
 	DIFF,
 	ERR,
 	INDEX,
+	PATCH,
 	SUMMARY,
 	TAG,
 	TAGS,
 	TREE,
 	RSS,
-	ACTIONS__MAX,
 };
 
 extern struct gotwebd	*gotwebd_env;
@@ -467,7 +464,7 @@ void sockets_socket_accept(int, short, void *);
 int sockets_privinit(struct gotwebd *, struct socket *);
 
 /* gotweb.c */
-void gotweb_get_navs(struct request *, struct gotweb_url *, int *,
+void gotweb_index_navs(struct request *, struct gotweb_url *, int *,
     struct gotweb_url *, int *);
 int gotweb_render_age(struct template *, time_t);
 const struct got_error *gotweb_init_transport(struct transport **);
@@ -495,6 +492,7 @@ int	gotweb_render_diff(struct template *);
 int	gotweb_render_branches(struct template *, struct got_reflist_head *);
 int	gotweb_render_summary(struct template *);
 int	gotweb_render_blame(struct template *);
+int	gotweb_render_patch(struct template *);
 int	gotweb_render_rss(struct template *);
 
 /* parse.y */
@@ -518,10 +516,10 @@ const struct got_error *got_get_repo_commits(struct request *, size_t);
 const struct got_error *got_get_repo_tags(struct request *, size_t);
 const struct got_error *got_get_repo_heads(struct request *);
 const struct got_error *got_open_diff_for_output(FILE **, struct request *);
-int got_output_repo_tree(struct request *,
+int got_output_repo_tree(struct request *, char **,
     int (*)(struct template *, struct got_tree_entry *));
 const struct got_error *got_open_blob_for_output(struct got_blob_object **,
-    int *, int *, struct request *);
+    int *, int *, struct request *, const char *, const char *, const char *);
 int got_output_blob_by_lines(struct template *, struct got_blob_object *,
     int (*)(struct template *, const char *, size_t));
 const struct got_error *got_output_file_blame(struct request *,
