@@ -14,6 +14,8 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include "got_compat.h"
+
 #include <sys/mman.h>
 #include <sys/queue.h>
 #include <sys/types.h>
@@ -113,8 +115,8 @@ request_packed_object_raw(uint8_t **outbuf, off_t *size, size_t *hdrlen,
 	struct imsgbuf *ibuf = pack->privsep_child->ibuf;
 	int outfd_child;
 
-	 err = pack_child_send_tempfiles(ibuf, pack);
-	 if (err)
+	err = pack_child_send_tempfiles(ibuf, pack);
+	if (err)
 		return err;
 
 	outfd_child = dup(outfd);
@@ -813,8 +815,8 @@ request_packed_blob(uint8_t **outbuf, size_t *size, size_t *hdrlen, int outfd,
 	struct imsgbuf *ibuf = pack->privsep_child->ibuf;
 	int outfd_child;
 
-	 err = pack_child_send_tempfiles(ibuf, pack);
-	 if (err)
+	err = pack_child_send_tempfiles(ibuf, pack);
+	if (err)
 		return err;
 
 	outfd_child = dup(outfd);
