@@ -30,7 +30,7 @@ got_opentempfd(void)
 	char name[PATH_MAX];
 	int fd;
 
-	if (strlcpy(name, GOT_TMPDIR_STR "/got.XXXXXXXX", sizeof(name))
+	if (strlcpy(name, GOT_TMPDIR_STR "/got.XXXXXXXXXX", sizeof(name))
 	    >= sizeof(name))
 		return -1;
 
@@ -72,7 +72,7 @@ got_opentemp_named(char **path, FILE **outfile, const char *basepath,
 
 	*outfile = NULL;
 
-	if (asprintf(path, "%s-XXXXXX%s", basepath, suffix) == -1) {
+	if (asprintf(path, "%s-XXXXXXXXXX%s", basepath, suffix) == -1) {
 		*path = NULL;
 		return got_error_from_errno("asprintf");
 	}
@@ -88,6 +88,7 @@ got_opentemp_named(char **path, FILE **outfile, const char *basepath,
 	*outfile = fdopen(fd, "w+");
 	if (*outfile == NULL) {
 		err = got_error_from_errno2("fdopen", *path);
+		close(fd);
 		free(*path);
 		*path = NULL;
 	}
@@ -104,7 +105,7 @@ got_opentemp_named_fd(char **path, int *outfd, const char *basepath,
 
 	*outfd = -1;
 
-	if (asprintf(path, "%s-XXXXXX%s", basepath, suffix) == -1) {
+	if (asprintf(path, "%s-XXXXXXXXXX%s", basepath, suffix) == -1) {
 		*path = NULL;
 		return got_error_from_errno("asprintf");
 	}
