@@ -214,7 +214,6 @@ got_privsep_flush_imsg(struct imsgbuf *ibuf)
 const struct got_error *
 got_privsep_send_stop(int fd)
 {
-	const struct got_error *err = NULL;
 	struct imsgbuf ibuf;
 
 	imsg_init(&ibuf, fd);
@@ -222,8 +221,7 @@ got_privsep_send_stop(int fd)
 	if (imsg_compose(&ibuf, GOT_IMSG_STOP, 0, 0, -1, NULL, 0) == -1)
 		return got_error_from_errno("imsg_compose STOP");
 
-	err = flush_imsg(&ibuf);
-	return err;
+	return flush_imsg(&ibuf);
 }
 
 const struct got_error *
@@ -241,9 +239,14 @@ const struct got_error *
 got_privsep_send_raw_obj_req(struct imsgbuf *ibuf, int fd,
     struct got_object_id *id)
 {
+	const struct got_error *err;
+
 	if (imsg_compose(ibuf, GOT_IMSG_RAW_OBJECT_REQUEST, 0, 0, fd,
-	    id, sizeof(*id)) == -1)
-		return got_error_from_errno("imsg_compose RAW_OBJECT_REQUEST");
+	    id, sizeof(*id)) == -1) {
+		err = got_error_from_errno("imsg_compose RAW_OBJECT_REQUEST");
+		close(fd);
+		return err;
+	}
 
 	return flush_imsg(ibuf);
 }
@@ -267,7 +270,6 @@ const struct got_error *
 got_privsep_send_raw_obj(struct imsgbuf *ibuf, off_t size, size_t hdrlen,
     uint8_t *data)
 {
-	const struct got_error *err = NULL;
 	struct got_imsg_raw_obj iobj;
 	size_t len = sizeof(iobj);
 	struct ibuf *wbuf;
@@ -280,10 +282,8 @@ got_privsep_send_raw_obj(struct imsgbuf *ibuf, off_t size, size_t hdrlen,
 		len += (size_t)size + hdrlen;
 
 	wbuf = imsg_create(ibuf, GOT_IMSG_RAW_OBJECT, 0, 0, len);
-	if (wbuf == NULL) {
-		err = got_error_from_errno("imsg_create RAW_OBJECT");
-		return err;
-	}
+	if (wbuf == NULL)
+		return got_error_from_errno("imsg_create RAW_OBJECT");
 
 	if (imsg_add(wbuf, &iobj, sizeof(iobj)) == -1)
 		return got_error_from_errno("imsg_add RAW_OBJECT");
@@ -293,9 +293,7 @@ got_privsep_send_raw_obj(struct imsgbuf *ibuf, off_t size, size_t hdrlen,
 			return got_error_from_errno("imsg_add RAW_OBJECT");
 	}
 
-	wbuf->fd = -1;
 	imsg_close(ibuf, wbuf);
-
 	return flush_imsg(ibuf);
 }
 
@@ -388,6 +386,7 @@ const struct got_error *
 got_privsep_send_tree_req(struct imsgbuf *ibuf, int fd,
     struct got_object_id *id, int pack_idx)
 {
+	const struct got_error *err;
 	struct ibuf *wbuf;
 	size_t len;
 
@@ -397,18 +396,30 @@ got_privsep_send_tree_req(struct imsgbuf *ibuf, int fd,
 		len = sizeof(*id);
 
 	wbuf = imsg_create(ibuf, GOT_IMSG_TREE_REQUEST, 0, 0, len);
-	if (wbuf == NULL)
-		return got_error_from_errno("imsg_create TREE_REQUEST");
-
-	if (imsg_add(wbuf, id, sizeof(*id)) == -1)
-		return got_error_from_errno("imsg_add TREE_REQUEST");
-
-	if (pack_idx != -1) { /* tree is packed */
-		if (imsg_add(wbuf, &pack_idx, sizeof(pack_idx)) == -1)
-			return got_error_from_errno("imsg_add TREE_REQUEST");
+	if (wbuf == NULL) {
+		err = got_error_from_errno("imsg_create TREE_REQUEST");
+		if (fd != -1)
+			close(fd);
+		return err;
 	}
 
-	wbuf->fd = fd;
+	if (imsg_add(wbuf, id, sizeof(*id)) == -1) {
+		err = got_error_from_errno("imsg_add TREE_REQUEST");
+		if (fd != -1)
+			close(fd);
+		return err;
+	}
+
+	if (pack_idx != -1) { /* tree is packed */
+		if (imsg_add(wbuf, &pack_idx, sizeof(pack_idx)) == -1) {
+			err = got_error_from_errno("imsg_add TREE_REQUEST");
+			if (fd != -1)
+				close(fd);
+			return err;
+		}
+	}
+
+	ibuf_fd_set(wbuf, fd);
 	imsg_close(ibuf, wbuf);
 
 	return flush_imsg(ibuf);
@@ -418,6 +429,7 @@ const struct got_error *
 got_privsep_send_tag_req(struct imsgbuf *ibuf, int fd,
     struct got_object_id *id, int pack_idx)
 {
+	const struct got_error *err;
 	struct got_imsg_packed_object iobj;
 	void *data;
 	size_t len;
@@ -434,8 +446,12 @@ got_privsep_send_tag_req(struct imsgbuf *ibuf, int fd,
 	}
 
 	if (imsg_compose(ibuf, GOT_IMSG_TAG_REQUEST, 0, 0, fd, data, len)
-	    == -1)
-		return got_error_from_errno("imsg_compose TAG_REQUEST");
+	    == -1) {
+		err = got_error_from_errno("imsg_compose TAG_REQUEST");
+		if (fd != -1)
+			close(fd);
+		return err;
+	}
 
 	return flush_imsg(ibuf);
 }
@@ -558,8 +574,11 @@ got_privsep_send_fetch_req(struct imsgbuf *ibuf, int fd,
 	}
 
 	wbuf = imsg_create(ibuf, GOT_IMSG_FETCH_REQUEST, 0, 0, len);
-	if (wbuf == NULL)
-		return got_error_from_errno("imsg_create FETCH_HAVE_REF");
+	if (wbuf == NULL) {
+		err = got_error_from_errno("imsg_create FETCH_HAVE_REF");
+		close(fd);
+		return err;
+	}
 
 	memset(&fetchreq, 0, sizeof(fetchreq));
 	fetchreq.no_head = no_head;
@@ -579,22 +598,27 @@ got_privsep_send_fetch_req(struct imsgbuf *ibuf, int fd,
 	if (imsg_add(wbuf, &fetchreq, sizeof(fetchreq)) == -1)
 		return got_error_from_errno("imsg_add FETCH_REQUEST");
 	if (worktree_branch) {
-		if (imsg_add(wbuf, worktree_branch, worktree_branch_len) == -1)
-			return got_error_from_errno("imsg_add FETCH_REQUEST");
+		if (imsg_add(wbuf, worktree_branch, worktree_branch_len)
+		    == -1) {
+			err = got_error_from_errno("imsg_add FETCH_REQUEST");
+			close(fd);
+			return err;
+		}
 	}
 	if (remote_head) {
-		if (imsg_add(wbuf, remote_head, remote_head_len) == -1)
-			return got_error_from_errno("imsg_add FETCH_REQUEST");
+		if (imsg_add(wbuf, remote_head, remote_head_len) == -1) {
+			err = got_error_from_errno("imsg_add FETCH_REQUEST");
+			close(fd);
+			return err;
+		}
 	}
-	wbuf->fd = fd;
+	ibuf_fd_set(wbuf, fd);
+	fd = -1;
 	imsg_close(ibuf, wbuf);
 
 	err = flush_imsg(ibuf);
-	if (err) {
-		close(fd);
+	if (err)
 		return err;
-	}
-	fd = -1;
 
 	TAILQ_FOREACH(pe, have_refs, entry) {
 		const char *name = pe->path;
@@ -614,7 +638,6 @@ got_privsep_send_fetch_req(struct imsgbuf *ibuf, int fd,
 		if (imsg_add(wbuf, name, name_len) == -1)
 			return got_error_from_errno("imsg_add FETCH_HAVE_REF");
 
-		wbuf->fd = -1;
 		imsg_close(ibuf, wbuf);
 		err = flush_imsg(ibuf);
 		if (err)
@@ -640,7 +663,6 @@ got_privsep_send_fetch_req(struct imsgbuf *ibuf, int fd,
 			return got_error_from_errno(
 			    "imsg_add FETCH_WANTED_BRANCH");
 
-		wbuf->fd = -1;
 		imsg_close(ibuf, wbuf);
 		err = flush_imsg(ibuf);
 		if (err)
@@ -666,16 +688,13 @@ got_privsep_send_fetch_req(struct imsgbuf *ibuf, int fd,
 			return got_error_from_errno(
 			    "imsg_add FETCH_WANTED_REF");
 
-		wbuf->fd = -1;
 		imsg_close(ibuf, wbuf);
 		err = flush_imsg(ibuf);
 		if (err)
 			return err;
 	}
 
-
 	return NULL;
-
 }
 
 const struct got_error *
@@ -854,7 +873,6 @@ send_send_ref(const char *name, size_t name_len, struct got_object_id *id,
 	if (imsg_add(wbuf, name, name_len) == -1)
 		return got_error_from_errno("imsg_add SEND_REF");
 
-	wbuf->fd = -1;
 	imsg_close(ibuf, wbuf);
 	return flush_imsg(ibuf);
 }
@@ -884,10 +902,10 @@ got_privsep_send_send_req(struct imsgbuf *ibuf, int fd,
 		goto done;
 	}
 
+	fd = -1;
 	err = flush_imsg(ibuf);
 	if (err)
 		goto done;
-	fd = -1;
 
 	TAILQ_FOREACH(pe, have_refs, entry) {
 		const char *name = pe->path;
@@ -909,7 +927,6 @@ done:
 	if (fd != -1 && close(fd) == -1 && err == NULL)
 		err = got_error_from_errno("close");
 	return err;
-
 }
 
 const struct got_error *
@@ -1448,7 +1465,6 @@ send_tree_entries_batch(struct imsgbuf *ibuf,
 			return got_error_from_errno("imsg_add TREE_ENTRY");
 	}
 
-	wbuf->fd = -1;
 	imsg_close(ibuf, wbuf);
 	return NULL;
 }
@@ -2258,8 +2274,8 @@ got_privsep_recv_gitconfig_remotes(struct got_remote_repo **remotes,
 	switch (imsg.hdr.type) {
 	case GOT_IMSG_GITCONFIG_REMOTES:
 		if (datalen != sizeof(iremotes)) {
-			err = got_error(GOT_ERR_PRIVSEP_LEN);
-			break;
+			imsg_free(&imsg);
+			return got_error(GOT_ERR_PRIVSEP_LEN);
 		}
 		memcpy(&iremotes, imsg.data, sizeof(iremotes));
 		if (iremotes.nremotes == 0) {
@@ -2675,9 +2691,7 @@ got_privsep_send_commit_traversal_request(struct imsgbuf *ibuf,
 		return got_error_from_errno("imsg_add "
 		    "COMMIT_TRAVERSAL_REQUEST");
 
-	wbuf->fd = -1;
 	imsg_close(ibuf, wbuf);
-
 	return flush_imsg(ibuf);
 }
 
@@ -2780,7 +2794,6 @@ got_privsep_send_enumerated_tree(size_t *totlen, struct imsgbuf *ibuf,
 	if (imsg_add(wbuf, path, path_len) == -1)
 		return got_error_from_errno("imsg_add ENUMERATED_TREE");
 
-	wbuf->fd = -1;
 	imsg_close(ibuf, wbuf);
 
 	if (entries) {
@@ -2842,7 +2855,6 @@ got_privsep_send_enumerated_commit(struct imsgbuf *ibuf,
 	if (imsg_add(wbuf, &mtime, sizeof(mtime)) == -1)
 		return got_error_from_errno("imsg_add ENUMERATED_COMMIT");
 
-	wbuf->fd = -1;
 	imsg_close(ibuf, wbuf);
 	/* Don't flush yet, tree entries or ENUMERATION_DONE will follow. */
 	return NULL;
@@ -3172,7 +3184,6 @@ send_idlist(struct imsgbuf *ibuf, struct got_object_id **ids, size_t nids)
 			return got_error_from_errno("imsg_add OBJ_ID_LIST");
 	}
 
-	wbuf->fd = -1;
 	imsg_close(ibuf, wbuf);
 
 	return flush_imsg(ibuf);
@@ -3309,7 +3320,6 @@ got_privsep_send_reused_deltas(struct imsgbuf *ibuf,
 			return got_error_from_errno("imsg_add REUSED_DELTAS");
 	}
 
-	wbuf->fd = -1;
 	imsg_close(ibuf, wbuf);
 
 	return flush_imsg(ibuf);
@@ -3446,9 +3456,7 @@ send_painted_commits(struct got_object_id_queue *ids, int *nids,
 		got_object_qid_free(qid);
 	}
 
-	wbuf->fd = -1;
 	imsg_close(ibuf, wbuf);
-
 	return flush_imsg(ibuf);
 }
 
@@ -3502,22 +3510,32 @@ got_privsep_recv_painted_commits(struct got_object_id_queue *new_ids,
 
 	for (;;) {
 		err = got_privsep_recv_imsg(&imsg, ibuf, 0);
-		if (err)
+		if (err){
+			imsg_free(&imsg);
 			return err;
+		}
 
 		datalen = imsg.hdr.len - IMSG_HEADER_SIZE;
-		if (imsg.hdr.type == GOT_IMSG_COMMIT_PAINTING_DONE)
-			break;
-		if (imsg.hdr.type != GOT_IMSG_PAINTED_COMMITS)
+		if (imsg.hdr.type == GOT_IMSG_COMMIT_PAINTING_DONE) {
+			imsg_free(&imsg);
+			return NULL;
+		}
+		if (imsg.hdr.type != GOT_IMSG_PAINTED_COMMITS){
+			imsg_free(&imsg);
 			return got_error(GOT_ERR_PRIVSEP_MSG);
+		}
 
-		if (datalen < sizeof(icommits))
+		if (datalen < sizeof(icommits)){
+			imsg_free(&imsg);
 			return got_error(GOT_ERR_PRIVSEP_LEN);
+		}
 		memcpy(&icommits, imsg.data, sizeof(icommits));
 		if (icommits.ncommits * sizeof(icommit) < icommits.ncommits ||
 		    datalen < sizeof(icommits) +
-		    icommits.ncommits * sizeof(icommit))
+		    icommits.ncommits * sizeof(icommit)){
+			imsg_free(&imsg);
 			return got_error(GOT_ERR_PRIVSEP_LEN);
+		}
 
 		for (i = 0; i < icommits.ncommits; i++) {
 			memcpy(&icommit,
@@ -3543,9 +3561,9 @@ got_privsep_recv_painted_commits(struct got_object_id_queue *new_ids,
 		}
 
 		imsg_free(&imsg);
+		if (err)
+			return err;
 	}
-
-	return err;
 }
 
 const struct got_error *
