@@ -408,7 +408,6 @@ send_capability(struct got_capability *capa, struct imsgbuf *ibuf)
 			return got_error_from_errno("imsg_add CAPABILITY");
 	}
 
-	wbuf->fd = -1;
 	imsg_close(ibuf, wbuf);
 
 	return NULL;
@@ -666,6 +665,7 @@ recv_done(int *packfd, int outfd, struct imsgbuf *ibuf, int chattygot)
 {
 	const struct got_error *err;
 	struct imsg imsg;
+	int fd;
 
 	*packfd = -1;
 
@@ -686,8 +686,9 @@ recv_done(int *packfd, int outfd, struct imsgbuf *ibuf, int chattygot)
 			err = gotd_imsg_recv_error(NULL, &imsg);
 			break;
 		case GOTD_IMSG_PACKFILE_PIPE:
-			if (imsg.fd != -1)
-				*packfd = imsg.fd;
+			fd = imsg_get_fd(&imsg);
+			if (fd != -1)
+				*packfd = fd;
 			else
 				err = got_error(GOT_ERR_PRIVSEP_NO_FD);
 			break;
@@ -1024,7 +1025,6 @@ recv_ref_update(int *report_status, int outfd, struct imsgbuf *ibuf,
 		return got_error_from_errno("imsg_add REF_UPDATE");
 	if (imsg_add(wbuf, refname, iref.name_len) == -1)
 		return got_error_from_errno("imsg_add REF_UPDATE");
-	wbuf->fd = -1;
 	imsg_close(ibuf, wbuf);
 
 	err = gotd_imsg_flush(ibuf);
@@ -1072,10 +1072,10 @@ recv_packfile(struct imsg *imsg, int infd)
 	if (datalen != 0)
 		return got_error(GOT_ERR_PRIVSEP_MSG);
 
-	if (imsg->fd == -1)
+	packfd = imsg_get_fd(imsg);
+	if (packfd == -1)
 		return got_error(GOT_ERR_PRIVSEP_NO_FD);
 
-	packfd = imsg->fd;
 	while (!pack_done) {
 		ssize_t r = 0;
 
@@ -1105,10 +1105,12 @@ recv_packfile(struct imsg *imsg, int infd)
 			/* Detect gotd(8) closing the pack pipe when done. */
 			err = got_poll_fd(packfd, 0, 1);
 			if (err) {
-				if (err->code != GOT_ERR_EOF)
+				if (err->code != GOT_ERR_TIMEOUT &&
+				    err->code != GOT_ERR_EOF)
 					break;
+				if (err->code == GOT_ERR_EOF)
+					pack_done = 1;
 				err = NULL;
-				pack_done = 1;
 			}
 		} else {
 			/* Write pack data and/or detect pipe being closed. */
