@@ -14,6 +14,8 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include "got_compat.h"
+
 #include <sys/types.h>
 #include <sys/queue.h>
 #include <sys/uio.h>
@@ -33,8 +35,6 @@
 #include <zlib.h>
 #include <err.h>
 
-#include "got_compat.h"
-
 #include "got_error.h"
 #include "got_object.h"
 #include "got_path.h"
@@ -260,7 +260,6 @@ send_fetch_symrefs(struct imsgbuf *ibuf, struct got_pathlist_head *symrefs)
 			return got_error_from_errno("imsg_add FETCH_SYMREFS");
 	}
 
-	wbuf->fd = -1;
 	imsg_close(ibuf, wbuf);
 	return got_privsep_flush_imsg(ibuf);
 }
@@ -286,7 +285,6 @@ send_fetch_ref(struct imsgbuf *ibuf, struct got_object_id *refid,
 	if (imsg_add(wbuf, refname, reflen) == -1)
 		return got_error_from_errno("imsg_add FETCH_REF");
 
-	wbuf->fd = -1;
 	imsg_close(ibuf, wbuf);
 	return got_privsep_flush_imsg(ibuf);
 }
@@ -350,14 +348,14 @@ fetch_pack(int fd, int packfd, uint8_t *pack_sha1,
 	struct got_pathlist_entry *pe;
 	int sent_my_capabilites = 0, have_sidebands = 0;
 	int found_branch = 0;
-	SHA1_CTX sha1_ctx;
+	struct got_hash ctx;
 	uint8_t sha1_buf[SHA1_DIGEST_LENGTH];
 	size_t sha1_buf_len = 0;
 	ssize_t w;
 	struct got_ratelimit rl;
 
 	TAILQ_INIT(&symrefs);
-	SHA1Init(&sha1_ctx);
+	got_hash_init(&ctx, GOT_HASH_SHA1);
 	got_ratelimit_init(&rl, 0, 500);
 
 	have = malloc(refsz * sizeof(have[0]));
@@ -761,7 +759,8 @@ fetch_pack(int fd, int packfd, uint8_t *pack_sha1,
 				    sha1_buf_len + r > SHA1_DIGEST_LENGTH) {
 					size_t nshift = MIN(sha1_buf_len + r -
 					    SHA1_DIGEST_LENGTH, sha1_buf_len);
-					SHA1Update(&sha1_ctx, sha1_buf, nshift);
+					got_hash_update(&ctx, sha1_buf,
+					    nshift);
 					memmove(sha1_buf, sha1_buf + nshift,
 					    sha1_buf_len - nshift);
 					sha1_buf_len -= nshift;
@@ -775,7 +774,7 @@ fetch_pack(int fd, int packfd, uint8_t *pack_sha1,
 				 * Mix in previously buffered bytes which
 				 * are not part of the checksum after all.
 				 */
-				SHA1Update(&sha1_ctx, sha1_buf, r);
+				got_hash_update(&ctx, sha1_buf, r);
 
 				/* Update potential checksum buffer. */
 				memmove(sha1_buf, sha1_buf + r,
@@ -784,10 +783,10 @@ fetch_pack(int fd, int packfd, uint8_t *pack_sha1,
 			}
 		} else {
 			/* Mix in any previously buffered bytes. */
-			SHA1Update(&sha1_ctx, sha1_buf, sha1_buf_len);
+			got_hash_update(&ctx, sha1_buf, sha1_buf_len);
 
 			/* Mix in bytes read minus potential checksum bytes. */
-			SHA1Update(&sha1_ctx, buf, r - SHA1_DIGEST_LENGTH);
+			got_hash_update(&ctx, buf, r - SHA1_DIGEST_LENGTH);
 
 			/* Buffer potential checksum bytes. */
 			memcpy(sha1_buf, buf + r - SHA1_DIGEST_LENGTH,
@@ -819,7 +818,7 @@ fetch_pack(int fd, int packfd, uint8_t *pack_sha1,
 	if (err)
 		goto done;
 
-	SHA1Final(pack_sha1, &sha1_ctx);
+	got_hash_final(&ctx, pack_sha1);
 	if (sha1_buf_len != SHA1_DIGEST_LENGTH ||
 	    memcmp(pack_sha1, sha1_buf, sha1_buf_len) != 0) {
 		err = got_error_msg(GOT_ERR_BAD_PACKFILE,
@@ -903,7 +902,7 @@ main(int argc, char **argv)
 		goto done;
 	}
 	memcpy(&fetch_req, imsg.data, sizeof(fetch_req));
-	fetchfd = imsg.fd;
+	fetchfd = imsg_get_fd(&imsg);
 
 	if (datalen != sizeof(fetch_req) +
 	    fetch_req.worktree_branch_len + fetch_req.remote_head_len) {
@@ -1080,7 +1079,7 @@ main(int argc, char **argv)
 		err = got_error(GOT_ERR_PRIVSEP_LEN);
 		goto done;
 	}
-	packfd = imsg.fd;
+	packfd = imsg_get_fd(&imsg);
 
 	err = fetch_pack(fetchfd, packfd, pack_sha1, &have_refs,
 	    fetch_req.fetch_all_branches, &wanted_branches,
