@@ -34,6 +34,7 @@
 #include "got_object.h"
 
 #include "got_lib_delta.h"
+#include "got_lib_hash.h"
 #include "got_lib_inflate.h"
 #include "got_lib_object.h"
 #include "got_lib_object_parse.h"
@@ -82,22 +83,21 @@ main(int argc, char *argv[])
 	for (;;) {
 		struct imsg imsg, imsg_outfd;
 		FILE *f = NULL;
+		int fd = -1, outfd = -1;
 		size_t size;
 		struct got_object *obj = NULL;
 		uint8_t *buf = NULL;
 		struct got_object_id id;
 		struct got_object_id expected_id;
 		struct got_inflate_checksum csum;
-		SHA1_CTX sha1_ctx;
+		struct got_hash ctx;
 
-		SHA1Init(&sha1_ctx);
+		got_hash_init(&ctx, GOT_HASH_SHA1);
 		memset(&csum, 0, sizeof(csum));
-		csum.output_sha1 = &sha1_ctx;
+		csum.output_ctx = &ctx;
 
 		memset(&imsg, 0, sizeof(imsg));
-		imsg.fd = -1;
 		memset(&imsg_outfd, 0, sizeof(imsg_outfd));
-		imsg_outfd.fd = -1;
 
 		if (sigint_received) {
 			err = got_error(GOT_ERR_CANCELLED);
@@ -126,7 +126,8 @@ main(int argc, char *argv[])
 		}
 		memcpy(&expected_id, imsg.data, sizeof(expected_id));
 
-		if (imsg.fd == -1) {
+		fd = imsg_get_fd(&imsg);
+		if (fd == -1) {
 			err = got_error(GOT_ERR_PRIVSEP_NO_FD);
 			goto done;
 		}
@@ -151,25 +152,27 @@ main(int argc, char *argv[])
 			err = got_error(GOT_ERR_PRIVSEP_LEN);
 			goto done;
 		}
-		if (imsg_outfd.fd == -1) {
+		outfd = imsg_get_fd(&imsg_outfd);
+		if (outfd == -1) {
 			err = got_error(GOT_ERR_PRIVSEP_NO_FD);
 			goto done;
 		}
 
-		err = got_object_read_header(&obj, imsg.fd);
+		err = got_object_read_header(&obj, fd);
 		if (err)
 			goto done;
 
-		if (lseek(imsg.fd, SEEK_SET, 0) == -1) {
+		if (lseek(fd, SEEK_SET, 0) == -1) {
 			err = got_error_from_errno("lseek");
 			goto done;
 		}
 
-		f = fdopen(imsg.fd, "rb");
+		f = fdopen(fd, "rb");
 		if (f == NULL) {
 			err = got_error_from_errno("fdopen");
 			goto done;
 		}
+		fd = -1;
 
 		if (obj->size + obj->hdrlen <=
 		    GOT_PRIVSEP_INLINE_BLOB_DATA_MAX) {
@@ -177,11 +180,11 @@ main(int argc, char *argv[])
 			if (err)
 				goto done;
 		} else {
-			err = got_inflate_to_fd(&size, f, &csum, imsg_outfd.fd);
+			err = got_inflate_to_fd(&size, f, &csum, outfd);
 			if (err)
 				goto done;
 		}
-		SHA1Final(id.sha1, &sha1_ctx);
+		got_hash_final_object_id(&ctx, &id);
 		if (got_object_id_cmp(&expected_id, &id) != 0) {
 			err = got_error_checksum(&expected_id);
 			goto done;
@@ -195,17 +198,12 @@ main(int argc, char *argv[])
 		err = got_privsep_send_blob(&ibuf, size, obj->hdrlen, buf);
 done:
 		free(buf);
-		if (f) {
-			if (fclose(f) == EOF && err == NULL)
-				err = got_error_from_errno("fclose");
-		} else if (imsg.fd != -1) {
-			if (close(imsg.fd) == -1 && err == NULL)
-				err = got_error_from_errno("close");
-		}
-		if (imsg_outfd.fd != -1) {
-			if (close(imsg_outfd.fd) == -1 && err == NULL)
-				err = got_error_from_errno("close");
-		}
+		if (f && fclose(f) == EOF && err == NULL)
+			err = got_error_from_errno("fclose");
+		if (fd != -1 && close(fd) == -1 && err == NULL)
+			err = got_error_from_errno("close");
+		if (outfd != -1 && close(outfd) == -1 && err == NULL)
+			err = got_error_from_errno("close");
 
 		imsg_free(&imsg);
 		imsg_free(&imsg_outfd);
