@@ -34,6 +34,7 @@
  * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
+#include "got_compat.h"
 
 #include <sys/types.h>
 #include <sys/queue.h>
@@ -51,8 +52,6 @@
 #include "got_error.h"
 #include "got_object.h"
 
-#include "got_compat.h"
-
 #include "got_lib_delta.h"
 #include "got_lib_object.h"
 #include "got_lib_privsep.h"
@@ -444,23 +443,29 @@ parse_hdr(char *s, int *done, struct got_imsg_patch_hunk *hdr)
 }
 
 static const struct got_error *
-send_line(const char *line)
+send_line(const char *line, size_t len)
 {
 	static const struct got_error *err = NULL;
-	char *p = NULL;
+	struct iovec iov[2];
+	int iovcnt = 0;
+
+	memset(&iov, 0, sizeof(iov));
 
 	if (*line != '+' && *line != '-' && *line != ' ' && *line != '\\') {
-		if (asprintf(&p, " %s", line) == -1)
-			return got_error_from_errno("asprintf");
-		line = p;
+		iov[iovcnt].iov_base = (void *)" ";
+		iov[iovcnt].iov_len = 1;
+		iovcnt++;
 	}
 
-	if (imsg_compose(&ibuf, GOT_IMSG_PATCH_LINE, 0, 0, -1,
-	    line, strlen(line) + 1) == -1)
+	iov[iovcnt].iov_base = (void *)line;
+	iov[iovcnt].iov_len = len;
+	iovcnt++;
+
+	if (imsg_composev(&ibuf, GOT_IMSG_PATCH_LINE, 0, 0, -1,
+	    iov, iovcnt) == -1)
 		err = got_error_from_errno(
 		    "imsg_compose GOT_IMSG_PATCH_LINE");
 
-	free(p);
 	return err;
 }
 
@@ -477,7 +482,7 @@ peek_special_line(FILE *fp)
 	}
 
 	if (ch == '\\') {
-		err = send_line("\\");
+		err = send_line("\\", 2);
 		if (err)
 			return err;
 	}
@@ -567,7 +572,7 @@ parse_hunk(FILE *fp, int *done)
 			goto done;
 		}
 
-		err = send_line(line);
+		err = send_line(line, linelen);
 		if (err)
 			goto done;
 
@@ -585,19 +590,12 @@ done:
 }
 
 static const struct got_error *
-read_patch(struct imsgbuf *ibuf, int fd)
+read_patch(struct imsgbuf *ibuf, FILE *fp)
 {
 	const struct got_error *err = NULL;
-	FILE *fp;
 	int git, patch_found = 0;
 	char *cid = NULL;
 
-	if ((fp = fdopen(fd, "r")) == NULL) {
-		err = got_error_from_errno("fdopen");
-		close(fd);
-		return err;
-	}
-
 	while ((err = patch_start(&git, &cid, fp)) == NULL) {
 		int done, next;
 
@@ -621,7 +619,6 @@ read_patch(struct imsgbuf *ibuf, int fd)
 	}
 
 done:
-	fclose(fp);
 	free(cid);
 
 	/* ignore trailing gibberish */
@@ -636,6 +633,8 @@ main(int argc, char **argv)
 {
 	const struct got_error *err = NULL;
 	struct imsg imsg;
+	FILE *fp = NULL;
+	int fd = -1;
 #if 0
 	static int attached;
 	while (!attached)
@@ -667,12 +666,24 @@ main(int argc, char **argv)
 	err = got_privsep_recv_imsg(&imsg, &ibuf, 0);
 	if (err)
 		goto done;
-	if (imsg.hdr.type != GOT_IMSG_PATCH_FILE || imsg.fd == -1) {
+	if (imsg.hdr.type != GOT_IMSG_PATCH_FILE) {
 		err = got_error(GOT_ERR_PRIVSEP_MSG);
 		goto done;
 	}
+	fd = imsg_get_fd(&imsg);
+	if (fd == -1) {
+		err = got_error(GOT_ERR_PRIVSEP_NO_FD);
+		goto done;
+	}
 
-	err = read_patch(&ibuf, imsg.fd);
+	fp = fdopen(fd, "r");
+	if (fp == NULL) {
+		err = got_error_from_errno("fdopen");
+		goto done;
+	}
+	fd = -1;
+
+	err = read_patch(&ibuf, fp);
 	if (err)
 		goto done;
 	if (imsg_compose(&ibuf, GOT_IMSG_PATCH_EOF, 0, 0, -1,
@@ -683,6 +694,10 @@ main(int argc, char **argv)
 	err = got_privsep_flush_imsg(&ibuf);
 done:
 	imsg_free(&imsg);
+	if (fd != -1 && close(fd) == -1 && err == NULL)
+		err = got_error_from_errno("close");
+	if (fp != NULL && fclose(fp) == EOF && err == NULL)
+		err = got_error_from_errno("fclose");
 	if (err != NULL) {
 		got_privsep_send_error(&ibuf, err);
 		err = NULL;
